#Internals for Open Firmware interface.


#Defines for an argument array structure.

	.set service, 0
	.set n_args, 4
	.set n_ret, 8
	.set arg0, 12


#The various (constant) strings that we use.

	cdata

#device names
chosen_str:	.string "/chosen"

#property names
stdin_str:	.string "stdin"
stdout_str:	.string "stdout"

#services
finddevice_str:	.string "finddevice"
getprop_str:	.string "getprop"
open_str:	.string "open"
close_str:	.string "close"
read_str:	.string "read"
write_str:	.string "write"
seek_str:	.string "seek"
exit_str:	.string "exit"

#Macro/routine to actually make the call to Open Firmware.
	#Note that, in addition to any return values in the
	#argument array, OF also returns a -1 in r3 if it failed
	#to call the requested service, or a 0 if it did make the
	#call.  However, I am currently ignoring this.

	.macro ofcall service_str
	liw r4,\service_str
	stw r4,service(r3)
	bl of_call
	twnei r3,0  #check for OF errors.
	.endm

	code

of_call:
	lwzi r4,r5,of_entry
	mtctr r4
	bctr


#Macro/routines to set n_args, n_ret, and clear return value (if
#any), for the various values of n_args and n_ret that we use.
#Also macro to get return value.

	.macro set_n n_args,n_ret
	#set n_args and n_ret.
	.if \n_args <> \n_ret
	li r4,\n_args
	li r5,\n_ret
	stw r4,n_args(r3)
	stw r5,n_ret(r3)
	.else
	li r4,\n_args
	stw r4,n_args(r3)
	stw r4,n_ret(r3)
	.endif
	#clear return value (if there is one).
	.if \n_ret
	li r4,0
	stw r4,arg0+\n_args<<2(r3)
	.endif
	.endm

	.macro get_retval n_args
	lwzi r3,r4,args+arg0+\n_args<<2
	.endm

	code

args_0_0:
	liw r3,args
	set_n 0,0
	blr

args_1_0:
	liw r4,args
	stw r3,arg0(r4)
	mr r3,r4
	set_n 1,0
	blr

args_1_1:
	liw r4,args
	stw r3,arg0(r4)
	mr r3,r4
	set_n 1,1
	blr

args_3_1:
	liw r6,args
	stw r3,arg0(r6)
	stw r4,arg0+4(r6)
	stw r5,arg0+8(r6)
	mr r3,r6
	set_n 3,1
	blr

args_4_1:
	liw r7,args
	stw r3,arg0(r7)
	stw r4,arg0+4(r7)
	stw r5,arg0+8(r7)
	stw r6,arg0+12(r7)
	mr r3,r7
	set_n 4,1
	blr


#stuff for Forth.
	
	cdata
qw_to_dv_table:
	.byte 00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07
	.byte 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f
	.byte 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17
	.byte 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f
	.byte 0x20,  '!',  '_',  '#',  '$',  '%',  '&',  '-'
	.byte  '(',  ')',  '*',  '}',  'w',  '[',  'v',  'z'
	.byte  '0',  '1',  '2',  '3',  '4',  '5',  '6',  '7'
	.byte  '8',  '9',  'S',  's',  'W',  ']',  'V',  'Z'
	.byte  '@',  'A',  'X',  'J',  'E',  '>',  'U',  'I'
	.byte  'D',  'C',  'H',  'T',  'N',  'M',  'B',  'R'
	.byte  'L',  '"',  'P',  'O',  'Y',  'G',  'K',  '<'
	.byte  'Q',  'F',  ':',  '/', '\\',  '=',  '^',  '{'
	.byte  '`',  'a',  'x',  'j',  'e',  '.',  'u',  'i'
	.byte  'd',  'c',  'h',  't',  'n',  'm',  'b',  'r'
	.byte  'l', '\'',  'p',  'o',  'y',  'g',  'k',  ','
	.byte  'q',  'f',  ';',  '?',  '|',  '+',  '~', 0x7f

	code

translate_to_dvorak:
	liw r6,qw_to_dv_table
	liw r5,input
	lwz r3,in_buf(r5)
	addi r3,r3,-1
	lwz r4,in_len(r5)
	mtctr r4
0:	lbzu r4,1(r3)
	lbzx r4,r6,r4
	stb r4,0(r3)
	bdnz 0b
	blr

echo_input:
	lwzi r3,r3,stdout
	liw r6,input
	lwz r4,in_buf(r6)
	lwz r5,in_len(r6)
	b write
