#OpenFirmware-specific code.

	.include "forth.inc"
	.include "of-internals.s"

	space

#ihandles
	.global stdin, stdout
stdin:	.int 0
stdout:	.int 0

#phandles
	.global chosen
chosen:	.int 0

#argument structure
	#To access an OF client service, you simply call the OF
	#entry point, with r3 pointing to an argument array
	#structure like the one below.
args:
	.int 0  #address of service string.
	.int 0  #number of arguments.
	.int 0  #number of return values.
	.int 0, 0, 0, 0, 0  #space for args and return values.

#Open Firmware entry point.
of_entry:	.int 0

#input buffer.
file_buf:	.int 0


	code

	#finddevice ( path -- phandle )
	.global finddevice
finddevice:
	prolog
	bl args_1_1
	ofcall finddevice_str
	get_retval 1
	epilog

	#getprop ( phandle name buf buflen -- len )
	.global getprop
getprop:
	prolog
	bl args_4_1
	ofcall getprop_str
	get_retval 4
	epilog

	#open ( path -- ihandle )
	.global open
open:
	prolog
	bl args_1_1
	ofcall open_str
	get_retval 1
	epilog


	#close ( ihandle -- )
	.global close
close:
	prolog
	bl args_1_0
	ofcall close_str
	epilog

	#read ( ihandle buf buflen -- len )
	.global read
read:
	prolog
	bl args_3_1
	ofcall read_str
	get_retval 3
	epilog

	#write ( ihandle buf buflen -- len )
	.global write
write:
	prolog
	bl args_3_1
	ofcall write_str
	get_retval 3
	epilog

	#seek ( ihandle pos.hi pos.lo -- status)
	.global seek
seek:
	prolog
	bl args_3_1
	ofcall seek_str
	get_retval 3
	epilog

	#exit ( -- )
	.global f_bye, exit
f_bye:
exit:
	prolog
	bl args_0_0
	ofcall exit_str
	epilog


#Forth interface code

	.global f_emit
f_emit:
	prolog
	lwzi r3,r3,stdout
	stw tos,-4(ds)    #store cell on top of stack,
	addi r4,ds,-1     #and get address of LSB.
	drop
	li r5,1           #one byte
	b write

	.global f_type
f_type:
	lwzi r3,r3,stdout
	nip r4
	mr r5,tos
	drop
	b write

	.global err_print
	#r4 = address, r5 = length
err_print:
	lwzi r3,r3,stdout
	b write

	.global f_refill
f_refill:
	prolog
0:	lwzi r3,r3,stdin
	lwzi r4,r4,file_buf
	li r5,1024
	bl read
	cmpwi r3,0  #wait for input.
	beq 0b
	liw r4,input
	lwzi r5,r5,file_buf
	stw r5,in_buf(r4)
	stw r3,in_len(r4)
	li r3,0
	stw r3,in_cur(r4)
	bl translate_to_dvorak
	bl echo_input
	epilog

	.global f_err
f_err:
	li r3,ERR_CODE
	b error


	.global _start
_start:
	#save address of OF entry point, (passed in r5, see the
	#OF PowerPC Processor binding).
	stwi r5,r3,of_entry

	#chosen = finddevice("/chosen");
	liw r3,chosen_str
	bl finddevice
	stwi r3,r4,chosen

	#stdin = getprop(chosen, "stdin");
	liw r4,stdin_str
	liw r5,stdin
	li r6,4
	bl getprop

	#stdout = getprop(chosen, "stdout");
	lwzi r3,r3,chosen
	liw r4,stdout_str
	liw r5,stdout
	li r6,4
	bl getprop

	#initialize and run the forth interpreter.
	bl init
	b run
